@@ -994,200 +994,56 @@ arrayops::is_finite(const eT* src, const uword n_elem)
 
 
 
-// TODO: this function is currently not used
 template<typename eT>
 arma_hot
 arma_pure
 inline
-typename get_pod_type<eT>::result
-arrayops::norm_1(const eT* src, const uword n_elem)
+bool
+arrayops::has_inf(const eT* src, const uword n_elem)
   {
-  typedef typename get_pod_type<eT>::result T;
+  uword j;
   
-  T acc = T(0);
-  
-  uword i,j;
-  
-  for(i=0, j=1; j<n_elem; i+=2, j+=2)
+  for(j=1; j<n_elem; j+=2)
     {
-    acc += std::abs(src[i]);
-    acc += std::abs(src[j]);
+    const eT val_i = (*src);  src++;
+    const eT val_j = (*src);  src++;
+    
+    if( arma_isinf(val_i) || arma_isinf(val_j) )  { return true; }
     }
   
-  if(i < n_elem)
+  if((j-1) < n_elem)
     {
-    acc += std::abs(src[i]);
+    if(arma_isinf(*src))  { return true; }
     }
   
-  return acc;
+  return false;
   }
 
 
 
-// TODO: this function is currently not used
 template<typename eT>
 arma_hot
 arma_pure
 inline
-eT
-arrayops::norm_2(const eT* src, const uword n_elem, const typename arma_not_cx<eT>::result* junk)
+bool
+arrayops::has_nan(const eT* src, const uword n_elem)
   {
-  arma_ignore(junk);
+  uword j;
   
-  eT acc = eT(0);
-  
-  uword i,j;
-  
-  for(i=0, j=1; j<n_elem; i+=2, j+=2)
+  for(j=1; j<n_elem; j+=2)
     {
-    const eT tmp_i = src[i];
-    const eT tmp_j = src[j];
+    const eT val_i = (*src);  src++;
+    const eT val_j = (*src);  src++;
     
-    acc += tmp_i * tmp_i;
-    acc += tmp_j * tmp_j;
+    if( arma_isnan(val_i) || arma_isnan(val_j) )  { return true; }
     }
   
-  if(i < n_elem)
+  if((j-1) < n_elem)
     {
-    const eT tmp_i = src[i];
-    
-    acc += tmp_i * tmp_i;
+    if(arma_isnan(*src))  { return true; }
     }
   
-  return std::sqrt(acc);
-  }
-
-
-
-// TODO: this function is currently not used
-template<typename T>
-arma_hot
-arma_pure
-inline
-T
-arrayops::norm_2(const std::complex<T>* src, const uword n_elem)
-  {
-  T acc = T(0);
-  
-  uword i,j;
-  
-  for(i=0, j=1; j<n_elem; i+=2, j+=2)
-    {
-    const T tmp_i = std::abs(src[i]);
-    const T tmp_j = std::abs(src[j]);
-    
-    acc += tmp_i * tmp_i;
-    acc += tmp_j * tmp_j;
-    }
-  
-  if(i < n_elem)
-    {
-    const T tmp_i = std::abs(src[i]);
-    
-    acc += tmp_i * tmp_i;
-    }
-  
-  return std::sqrt(acc);
-  }
-
-
-
-// TODO: this function is currently not used
-template<typename eT>
-arma_hot
-arma_pure
-inline
-typename get_pod_type<eT>::result
-arrayops::norm_k(const eT* src, const uword n_elem, const int k)
-  {
-  typedef typename get_pod_type<eT>::result T;
-  
-  T acc = T(0);
-  
-  uword i,j;
-  
-  for(i=0, j=1; j<n_elem; i+=2, j+=2)
-    {
-    acc += std::pow(std::abs(src[i]), k);
-    acc += std::pow(std::abs(src[j]), k);
-    }
-  
-  if(i < n_elem)
-    {
-    acc += std::pow(std::abs(src[i]), k);
-    }
-  
-  return std::pow(acc, T(1)/T(k));
-  }
-
-
-
-// TODO: this function is currently not used
-template<typename eT>
-arma_hot
-arma_pure
-inline
-typename get_pod_type<eT>::result
-arrayops::norm_max(const eT* src, const uword n_elem)
-  {
-  typedef typename get_pod_type<eT>::result T;
-  
-  T max_val = std::abs(src[0]);
-  
-  uword i,j;
-  
-  for(i=1, j=2; j<n_elem; i+=2, j+=2)
-    {
-    const T tmp_i = std::abs(src[i]);
-    const T tmp_j = std::abs(src[j]);
-    
-    if(max_val < tmp_i) { max_val = tmp_i; }
-    if(max_val < tmp_j) { max_val = tmp_j; }
-    }
-  
-  if(i < n_elem)
-    {
-    const T tmp_i = std::abs(src[i]);
-    
-    if(max_val < tmp_i) { max_val = tmp_i; }
-    }
-  
-  return max_val;
-  }
-
-
-
-// TODO: this function is currently not used
-template<typename eT>
-arma_hot
-arma_pure
-inline
-typename get_pod_type<eT>::result
-arrayops::norm_min(const eT* src, const uword n_elem)
-  {
-  typedef typename get_pod_type<eT>::result T;
-  
-  T min_val = std::abs(src[0]);
-  
-  uword i,j;
-  
-  for(i=1, j=2; j<n_elem; i+=2, j+=2)
-    {
-    const T tmp_i = std::abs(src[i]);
-    const T tmp_j = std::abs(src[j]);
-    
-    if(min_val > tmp_i) { min_val = tmp_i; }
-    if(min_val > tmp_j) { min_val = tmp_j; }
-    }
-  
-  if(i < n_elem)
-    {
-    const T tmp_i = std::abs(src[i]);
-    
-    if(min_val > tmp_i) { min_val = tmp_i; }
-    }
-  
-  return min_val;
+  return false;
   }
 
 
